@@ -20,6 +20,7 @@ Archive Get Command
 #include "common/wait.h"
 #include "config/config.h"
 #include "config/exec.h"
+#include "config/load.h"
 #include "info/infoArchive.h"
 #include "postgres/interface.h"
 #include "protocol/helper.h"
@@ -155,7 +156,7 @@ archiveGetFind(
                                 strNewFmt(STORAGE_REPO_ARCHIVE "/%s/%s", strZ(cacheArchive->archiveId), strZ(path)),
                                 .expression = strNewFmt(
                                     "^%s%s-[0-f]{40}" COMPRESS_TYPE_REGEXP "{0,1}$", strZ(strSubN(archiveFileRequest, 0, 24)),
-                                        walIsPartial(archiveFileRequest) ? WAL_SEGMENT_PARTIAL_EXT : ""));
+                                    walIsPartial(archiveFileRequest) ? WAL_SEGMENT_PARTIAL_EXT : ""));
                         }
                         // Else multiple files will be requested so cache list results
                         else
@@ -170,17 +171,17 @@ archiveGetFind(
                             {
                                 MEM_CONTEXT_BEGIN(lstMemContext(cacheArchive->pathList))
                                 {
-                                    cachePath = lstAdd(
-                                        cacheArchive->pathList,
-                                        &(ArchiveGetFindCachePath)
-                                        {
-                                            .path = strDup(path),
-                                            .fileList = storageListP(
-                                                storageRepoIdx(cacheRepo->repoIdx),
-                                                strNewFmt(STORAGE_REPO_ARCHIVE "/%s/%s", strZ(cacheArchive->archiveId), strZ(path)),
-                                                .expression = strNewFmt(
-                                                    "^%s[0-F]{8}-[0-f]{40}" COMPRESS_TYPE_REGEXP "{0,1}$", strZ(path))),
-                                        });
+                                    const ArchiveGetFindCachePath archiveGetFindCachePath =
+                                    {
+                                        .path = strDup(path),
+                                        .fileList = storageListP(
+                                            storageRepoIdx(cacheRepo->repoIdx),
+                                            strNewFmt(STORAGE_REPO_ARCHIVE "/%s/%s", strZ(cacheArchive->archiveId), strZ(path)),
+                                            .expression = strNewFmt(
+                                                "^%s[0-F]{8}-[0-f]{40}" COMPRESS_TYPE_REGEXP "{0,1}$", strZ(path))),
+                                    };
+
+                                    cachePath = lstAdd(cacheArchive->pathList, &archiveGetFindCachePath);
                                 }
                                 MEM_CONTEXT_END();
                             }
@@ -200,39 +201,40 @@ archiveGetFind(
                         {
                             MEM_CONTEXT_BEGIN(lstMemContext(getCheckResult->archiveFileMapList))
                             {
-                                lstAdd(
-                                    matchList,
-                                    &(ArchiveGetFile)
-                                    {
-                                        .file = strNewFmt(
-                                            "%s/%s/%s", strZ(cacheArchive->archiveId), strZ(path),
-                                            strZ(strLstGet(segmentList, segmentIdx))),
-                                        .repoIdx = cacheRepo->repoIdx,
-                                        .archiveId = cacheArchive->archiveId,
-                                        .cipherType = cacheRepo->cipherType,
-                                        .cipherPassArchive = cacheRepo->cipherPassArchive,
-                                    });
+                                const ArchiveGetFile archiveGetFile =
+                                {
+                                    .file = strNewFmt(
+                                        "%s/%s/%s", strZ(cacheArchive->archiveId), strZ(path),
+                                        strZ(strLstGet(segmentList, segmentIdx))),
+                                    .repoIdx = cacheRepo->repoIdx,
+                                    .archiveId = cacheArchive->archiveId,
+                                    .cipherType = cacheRepo->cipherType,
+                                    .cipherPassArchive = cacheRepo->cipherPassArchive,
+                                };
+
+                                lstAdd(matchList, &archiveGetFile);
                             }
                             MEM_CONTEXT_END();
                         }
                     }
                     // Else if not a WAL segment, see if it exists in the archiveId path
-                    else if (storageExistsP(
-                        storageRepoIdx(cacheRepo->repoIdx), strNewFmt(STORAGE_REPO_ARCHIVE "/%s/%s", strZ(cacheArchive->archiveId),
-                        strZ(archiveFileRequest))))
+                    else if (
+                        storageExistsP(
+                            storageRepoIdx(cacheRepo->repoIdx),
+                            strNewFmt(STORAGE_REPO_ARCHIVE "/%s/%s", strZ(cacheArchive->archiveId), strZ(archiveFileRequest))))
                     {
                         MEM_CONTEXT_BEGIN(lstMemContext(getCheckResult->archiveFileMapList))
                         {
-                            lstAdd(
-                                matchList,
-                                &(ArchiveGetFile)
-                                {
-                                    .file = strNewFmt("%s/%s", strZ(cacheArchive->archiveId), strZ(archiveFileRequest)),
-                                    .repoIdx = cacheRepo->repoIdx,
-                                    .archiveId = cacheArchive->archiveId,
-                                    .cipherType = cacheRepo->cipherType,
-                                    .cipherPassArchive = cacheRepo->cipherPassArchive,
-                                });
+                            const ArchiveGetFile archiveGetFile =
+                            {
+                                .file = strNewFmt("%s/%s", strZ(cacheArchive->archiveId), strZ(archiveFileRequest)),
+                                .repoIdx = cacheRepo->repoIdx,
+                                .archiveId = cacheArchive->archiveId,
+                                .cipherType = cacheRepo->cipherType,
+                                .cipherPassArchive = cacheRepo->cipherPassArchive,
+                            };
+
+                            lstAdd(matchList, &archiveGetFile);
                         }
                         MEM_CONTEXT_END();
                     }
@@ -263,7 +265,6 @@ archiveGetFind(
                 getCheckResult->warnList = strLstMove(fileWarnList, memContextCurrent());
             }
             MEM_CONTEXT_END();
-
         }
         // Else if a file was found
         else if (!lstEmpty(matchList))
@@ -317,7 +318,7 @@ archiveGetFind(
                         getCheckResult->errorFile = strDup(archiveFileRequest);
                         getCheckResult->errorMessage = strNewFmt(
                             "duplicates found for WAL segment %s:%s\n"
-                                "HINT: are multiple primaries archiving to this stanza?",
+                            "HINT: are multiple primaries archiving to this stanza?",
                             strZ(archiveFileRequest), strZ(message));
                         getCheckResult->warnList = strLstMove(fileWarnList, memContextCurrent());
                     }
@@ -362,6 +363,8 @@ archiveGetCheck(const StringList *archiveRequestList)
         FUNCTION_LOG_PARAM(STRING_LIST, archiveRequestList);
     FUNCTION_LOG_END();
 
+    FUNCTION_AUDIT_STRUCT();
+
     ASSERT(archiveRequestList != NULL);
     ASSERT(!strLstEmpty(archiveRequestList));
 
@@ -372,8 +375,16 @@ archiveGetCheck(const StringList *archiveRequestList)
         // List of warnings
         StringList *warnList = strLstNew();
 
-        // Get pg control info
-        PgControl controlInfo = pgControlFromFile(storagePg());
+        // Get pg_control info. Error if the invalid checkpoint written by restore is detected and backup_label is not present.
+        PgControl controlInfo = pgControlFromFile(storagePg(), cfgOptionStrNull(cfgOptPgVersionForce));
+
+        if (controlInfo.checkpoint == PG_CONTROL_CHECKPOINT_INVALID && !storageExistsP(storagePg(), STRDEF(PG_FILE_BACKUPLABEL)))
+        {
+            THROW(
+                FormatError,
+                "pg_control from backup is not valid without backup_label\n"
+                "HINT: was the backup_label file removed?");
+        }
 
         // Build list of repos/archiveIds where WAL may be found
         List *cacheRepoList = lstNewP(sizeof(ArchiveGetFindCacheRepo));
@@ -537,13 +548,12 @@ queueNeed(const String *walSegment, bool found, uint64_t queueSize, size_t walSe
 
     MEM_CONTEXT_TEMP_BEGIN()
     {
-        // Determine the first WAL segment for the async process to get.  If the WAL segment requested by
-        // PostgreSQL was not found then use that.  If the segment was found but the queue is not full then
-        // start with the next segment.
+        // Determine the first WAL segment for the async process to get. If the WAL segment requested by PostgreSQL was not found
+        // then use that. If the segment was found but the queue is not full then start with the next segment.
         const String *walSegmentFirst =
             found ? walSegmentNext(walSegment, walSegmentSize, pgVersion) : walSegment;
 
-        // Determine how many WAL segments should be in the queue.  The queue total must be at least 2 or it doesn't make sense to
+        // Determine how many WAL segments should be in the queue. The queue total must be at least 2 or it doesn't make sense to
         // have async turned on at all.
         unsigned int walSegmentQueueTotal = (unsigned int)(queueSize / walSegmentSize);
 
@@ -596,7 +606,7 @@ queueNeed(const String *walSegment, bool found, uint64_t queueSize, size_t walSe
 }
 
 /**********************************************************************************************************************************/
-int
+FN_EXTERN int
 cmdArchiveGet(void)
 {
     FUNCTION_LOG_VOID(logLevelDebug);
@@ -712,16 +722,13 @@ cmdArchiveGet(void)
                     }
                 }
 
-                // If the WAL segment has not already been found then start the async process to get it.  There's no point in
-                // forking the async process off more than once so track that as well.  Use an archive lock to prevent forking if
-                // the async process was launched by another process.
-                if (!forked && (!found || !queueFull)  &&
-                    lockAcquire(
-                        cfgOptionStr(cfgOptLockPath), cfgOptionStr(cfgOptStanza), cfgOptionStr(cfgOptExecId), cfgLockType(), 0,
-                        false))
+                // If the WAL segment has not already been found then start the async process to get it. There's no point in forking
+                // the async process off more than once so track that as well. Use an archive lock to prevent forking if the async
+                // process was launched by another process.
+                if (!forked && (!found || !queueFull) && lockAcquireP(.returnOnNoLock = true))
                 {
                     // Get control info
-                    PgControl pgControl = pgControlFromFile(storagePg());
+                    PgControl pgControl = pgControlFromFile(storagePg(), cfgOptionStrNull(cfgOptPgVersionForce));
 
                     // Create the queue
                     storagePathCreateP(storageSpoolWrite(), STORAGE_SPOOL_ARCHIVE_IN_STR);
@@ -736,8 +743,8 @@ cmdArchiveGet(void)
                     StringList *commandExec = cfgExecParam(cfgCmdArchiveGet, cfgCmdRoleAsync, optionReplace, true, false);
                     strLstInsert(commandExec, 0, cfgExe());
 
-                    // Clean the current queue using the list of WAL that we ideally want in the queue.  queueNeed()
-                    // will return the list of WAL needed to fill the queue and this will be passed to the async process.
+                    // Clean the current queue using the list of WAL that we ideally want in the queue. queueNeed() will return the
+                    // list of WAL needed to fill the queue and this will be passed to the async process.
                     const StringList *queue = queueNeed(
                         walSegment, found, cfgOptionUInt64(cfgOptArchiveGetQueueMax), pgControl.walSegmentSize,
                         pgControl.version);
@@ -754,7 +761,7 @@ cmdArchiveGet(void)
                     // Execute the async process
                     archiveAsyncExec(archiveModeGet, commandExec);
 
-                    // Mark the async process as forked so it doesn't get forked again.  A single run of the async process should be
+                    // Mark the async process as forked so it doesn't get forked again. A single run of the async process should be
                     // enough to do the job, running it again won't help anything.
                     forked = true;
                 }
@@ -776,8 +783,11 @@ cmdArchiveGet(void)
                 if (!foundOk)
                 {
                     THROW_FMT(
-                        ArchiveTimeoutError, "unable to get WAL file '%s' from the archive asynchronously after %s second(s)",
-                        strZ(walSegment), strZ(strNewDbl((double)cfgOptionInt64(cfgOptArchiveTimeout) / MSEC_PER_SEC)));
+                        ArchiveTimeoutError,
+                        "unable to get WAL file '%s' from the archive asynchronously after %s second(s)\n"
+                        "HINT: check '%s' for errors.",
+                        strZ(walSegment), strZ(strNewDbl((double)cfgOptionInt64(cfgOptArchiveTimeout) / MSEC_PER_SEC)),
+                        strZ(cfgLoadLogFileName(cfgCmdRoleAsync)));
                 }
                 // Else report that the WAL segment could not be found
                 else
@@ -839,7 +849,8 @@ typedef struct ArchiveGetAsyncData
     unsigned int archiveFileIdx;                                    // Current index in the list to be processed
 } ArchiveGetAsyncData;
 
-static ProtocolParallelJob *archiveGetAsyncCallback(void *data, unsigned int clientIdx)
+static ProtocolParallelJob *
+archiveGetAsyncCallback(void *const data, const unsigned int clientIdx)
 {
     FUNCTION_TEST_BEGIN();
         FUNCTION_TEST_PARAM_P(VOID, data);
@@ -887,10 +898,10 @@ static ProtocolParallelJob *archiveGetAsyncCallback(void *data, unsigned int cli
     }
     MEM_CONTEXT_TEMP_END();
 
-    FUNCTION_TEST_RETURN(result);
+    FUNCTION_TEST_RETURN(PROTOCOL_PARALLEL_JOB, result);
 }
 
-void
+FN_EXTERN void
 cmdArchiveGetAsync(void)
 {
     FUNCTION_LOG_VOID(logLevelDebug);
@@ -910,7 +921,7 @@ cmdArchiveGetAsync(void)
                 "get %u WAL file(s) from archive: %s%s",
                 strLstSize(cfgCommandParam()), strZ(strLstGet(cfgCommandParam(), 0)),
                 strLstSize(cfgCommandParam()) == 1 ?
-                    "" : strZ(strNewFmt("...%s", strZ(strLstGet(cfgCommandParam(), strLstSize(cfgCommandParam()) - 1)))));
+                    "" : zNewFmt("...%s", strZ(strLstGet(cfgCommandParam(), strLstSize(cfgCommandParam()) - 1))));
 
             // Check for archive files
             ArchiveGetCheckResult checkResult = archiveGetCheck(cfgCommandParam());
@@ -1006,7 +1017,7 @@ cmdArchiveGetAsync(void)
                                     archiveModeGet, walSegment, protocolParallelJobErrorCode(job),
                                     strNewFmt(
                                         "%s%s", strZ(protocolParallelJobErrorMessage(job)),
-                                        strSize(warning) == 0 ? "" : strZ(strNewFmt("\n%s", strZ(warning)))));
+                                        strSize(warning) == 0 ? "" : zNewFmt("\n%s", strZ(warning))));
                             }
 
                             protocolParallelJobFree(job);
