@@ -4,20 +4,21 @@ Archive Get Protocol Handler
 #ifndef COMMAND_ARCHIVE_GET_PROTOCOL_H
 #define COMMAND_ARCHIVE_GET_PROTOCOL_H
 
-#include "common/type/string.h"
-#include "common/type/variantList.h"
+#include "common/type/pack.h"
 #include "protocol/server.h"
 
-/***********************************************************************************************************************************
-Constants
-***********************************************************************************************************************************/
-#define PROTOCOL_COMMAND_ARCHIVE_GET                                "archiveGet"
-    STRING_DECLARE(PROTOCOL_COMMAND_ARCHIVE_GET_STR);
-
 /***********************************************************************************************************************************
 Functions
 ***********************************************************************************************************************************/
 // Process protocol requests
-bool archiveGetProtocol(const String *command, const VariantList *paramList, ProtocolServer *server);
+FN_EXTERN void archiveGetFileProtocol(PackRead *param, ProtocolServer *server);
+
+/***********************************************************************************************************************************
+Protocol commands for ProtocolServerHandler arrays passed to protocolServerProcess()
+***********************************************************************************************************************************/
+#define PROTOCOL_COMMAND_ARCHIVE_GET_FILE                           STRID5("ag-f", 0x36ce10)
+
+#define PROTOCOL_SERVER_HANDLER_ARCHIVE_GET_LIST                                                                                   \
+    {.command = PROTOCOL_COMMAND_ARCHIVE_GET_FILE, .handler = archiveGetFileProtocol},
 
 #endif
