@@ -18,6 +18,7 @@ Archive Push Command
 #include "common/wait.h"
 #include "config/config.h"
 #include "config/exec.h"
+#include "config/load.h"
 #include "info/infoArchive.h"
 #include "postgres/interface.h"
 #include "postgres/version.h"
@@ -40,45 +41,49 @@ Ready file extension constants
 Format the warning when a file is dropped
 ***********************************************************************************************************************************/
 static String *
-archivePushDropWarning(const String *walFile, uint64_t queueMax)
+archivePushDropWarning(const String *const walFile, const uint64_t queueMax)
 {
     FUNCTION_TEST_BEGIN();
         FUNCTION_TEST_PARAM(STRING, walFile);
         FUNCTION_TEST_PARAM(UINT64, queueMax);
     FUNCTION_TEST_END();
 
-    FUNCTION_TEST_RETURN(
-        strNewFmt("dropped WAL file '%s' because archive queue exceeded %s", strZ(walFile), strZ(strSizeFormat(queueMax))));
+    String *const size = strSizeFormat(queueMax);
+    String *const result = strNewFmt("dropped WAL file '%s' because archive queue exceeded %s", strZ(walFile), strZ(size));
+
+    strFree(size);
+
+    FUNCTION_TEST_RETURN(STRING, result);
 }
 
 /***********************************************************************************************************************************
 Determine if the WAL process list has become large enough to drop
 ***********************************************************************************************************************************/
 static bool
-archivePushDrop(const String *walPath, const StringList *const processList)
+archivePushDrop(const String *const walPath, const StringList *const processList)
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
         FUNCTION_LOG_PARAM(STRING, walPath);
         FUNCTION_LOG_PARAM(STRING_LIST, processList);
     FUNCTION_LOG_END();
 
+    ASSERT(walPath != NULL);
+    ASSERT(processList != NULL);
+
     const uint64_t queueMax = cfgOptionUInt64(cfgOptArchivePushQueueMax);
     uint64_t queueSize = 0;
-    bool result = false;
 
-    for (unsigned int processIdx = 0; processIdx < strLstSize(processList); processIdx++)
+    MEM_CONTEXT_TEMP_BEGIN()
     {
-        queueSize += storageInfoP(
-            storagePg(), strNewFmt("%s/%s", strZ(walPath), strZ(strLstGet(processList, processIdx)))).size;
-
-        if (queueSize > queueMax)
+        if (!strLstEmpty(processList))
         {
-            result = true;
-            break;
+            queueSize = storageInfoP(
+                storagePg(), strNewFmt("%s/%s", strZ(walPath), strZ(strLstGet(processList, 0)))).size * strLstSize(processList);
         }
     }
+    MEM_CONTEXT_TEMP_END();
 
-    FUNCTION_LOG_RETURN(BOOL, result);
+    FUNCTION_LOG_RETURN(BOOL, queueSize > queueMax);
 }
 
 /***********************************************************************************************************************************
@@ -108,9 +113,8 @@ archivePushReadyList(const String *walPath)
 
         for (unsigned int readyIdx = 0; readyIdx < strLstSize(readyListRaw); readyIdx++)
         {
-            strLstAdd(
-                result,
-                strSubN(strLstGet(readyListRaw, readyIdx), 0, strSize(strLstGet(readyListRaw, readyIdx)) - STATUS_EXT_READY_SIZE));
+            const String *const ready = strLstGet(readyListRaw, readyIdx);
+            strLstAddSub(result, ready, strSize(ready) - STATUS_EXT_READY_SIZE);
         }
 
         strLstMove(result, memContextPrior());
@@ -123,9 +127,9 @@ archivePushReadyList(const String *walPath)
 /***********************************************************************************************************************************
 Determine which WAL files need to be pushed to the archive when in async mode
 
-This is the heart of the "look ahead" functionality in async archiving.  Any files in the out directory that do not end in ok are
+This is the heart of the "look ahead" functionality in async archiving. Any files in the out directory that do not end in ok are
 removed and any ok files that do not have a corresponding ready file in archive_status (meaning it has been acknowledged by
-PostgreSQL) are removed.  Then all ready files that do not have a corresponding ok file (meaning it has already been processed) are
+PostgreSQL) are removed. Then all ready files that do not have a corresponding ok file (meaning it has already been processed) are
 returned for processing.
 ***********************************************************************************************************************************/
 static StringList *
@@ -156,7 +160,7 @@ archivePushProcessList(const String *walPath)
             const String *statusFile = strLstGet(statusList, statusIdx);
 
             if (strEndsWithZ(statusFile, STATUS_EXT_OK))
-                strLstAdd(okList, strSubN(statusFile, 0, strSize(statusFile) - STATUS_EXT_OK_SIZE));
+                strLstAddSub(okList, statusFile, strSize(statusFile) - STATUS_EXT_OK_SIZE);
             else
             {
                 storageRemoveP(
@@ -189,7 +193,7 @@ archivePushProcessList(const String *walPath)
 /***********************************************************************************************************************************
 Check that pg_control and archive.info match and get the archive id and archive cipher passphrase (if present)
 
-As much information as possible is collected here so that async archiving has as little work as possible to do for each file.  Sync
+As much information as possible is collected here so that async archiving has as little work as possible to do for each file. Sync
 archiving does not benefit but it makes sense to use the same function.
 ***********************************************************************************************************************************/
 typedef struct ArchivePushCheckResult
@@ -207,6 +211,8 @@ archivePushCheck(bool pgPathSet)
         FUNCTION_LOG_PARAM(BOOL, pgPathSet);
     FUNCTION_LOG_END();
 
+    FUNCTION_AUDIT_STRUCT();
+
     ArchivePushCheckResult result = {.repoList = lstNewP(sizeof(ArchivePushFileRepoData)), .errorList = strLstNew()};
 
     MEM_CONTEXT_TEMP_BEGIN()
@@ -216,7 +222,7 @@ archivePushCheck(bool pgPathSet)
         if (pgPathSet)
         {
             // Get info from pg_control
-            PgControl pgControl = pgControlFromFile(storagePg());
+            PgControl pgControl = pgControlFromFile(storagePg(), cfgOptionStrNull(cfgOptPgVersionForce));
             result.pgVersion = pgControl.version;
             result.pgSystemId = pgControl.systemId;
         }
@@ -250,7 +256,7 @@ archivePushCheck(bool pgPathSet)
                             ArchiveMismatchError,
                             "%s version %s, system-id %" PRIu64 " do not match %s stanza version %s, system-id %" PRIu64
                             "\nHINT: are you archiving to the correct stanza?",
-                            pgPathSet ? PG_NAME : strZ(strNewFmt("%s stanza", cfgOptionGroupName(cfgOptGrpRepo, 0))),
+                            pgPathSet ? PG_NAME : zNewFmt("%s stanza", cfgOptionGroupName(cfgOptGrpRepo, 0)),
                             strZ(pgVersionToStr(result.pgVersion)), result.pgSystemId,
                             cfgOptionGroupName(cfgOptGrpRepo, repoIdx), strZ(pgVersionToStr(archiveInfo.version)),
                             archiveInfo.systemId);
@@ -262,24 +268,23 @@ archivePushCheck(bool pgPathSet)
                     result.pgVersion = archiveInfo.version;
                     result.pgSystemId = archiveInfo.systemId;
 
-                    lstAdd(
-                        result.repoList,
-                        &(ArchivePushFileRepoData)
-                        {
-                            .repoIdx = repoIdx,
-                            .archiveId = strDup(archiveId),
-                            .cipherType = repoCipherType,
-                            .cipherPass = strDup(infoArchiveCipherPass(info)),
-                        });
+                    const ArchivePushFileRepoData archivePushFileRepoData =
+                    {
+                        .repoIdx = repoIdx,
+                        .archiveId = strDup(archiveId),
+                        .cipherType = repoCipherType,
+                        .cipherPass = strDup(infoArchiveCipherPass(info)),
+                    };
+
+                    lstAdd(result.repoList, &archivePushFileRepoData);
                 }
                 MEM_CONTEXT_PRIOR_END();
             }
             CATCH_ANY()
             {
-                strLstAdd(
-                    result.errorList,
-                    strNewFmt(
-                        "%s: [%s] %s", cfgOptionGroupName(cfgOptGrpRepo, repoIdx), errorTypeName(errorType()), errorMessage()));
+                strLstAddFmt(
+                    result.errorList, "%s: [%s] %s", cfgOptionGroupName(cfgOptGrpRepo, repoIdx), errorTypeName(errorType()),
+                    errorMessage());
             }
             TRY_END();
         }
@@ -298,7 +303,7 @@ archivePushCheck(bool pgPathSet)
 }
 
 /**********************************************************************************************************************************/
-void
+FN_EXTERN void
 cmdArchivePush(void)
 {
     FUNCTION_LOG_VOID(logLevelDebug);
@@ -342,17 +347,14 @@ cmdArchivePush(void)
 
             do
             {
-                // Check if the WAL segment has been pushed.  Errors will not be thrown on the first try to allow the async process
+                // Check if the WAL segment has been pushed. Errors will not be thrown on the first try to allow the async process
                 // a chance to fix them.
                 pushed = archiveAsyncStatus(archiveModePush, archiveFile, throwOnError, true);
 
-                // If the WAL segment has not already been pushed then start the async process to push it.  There's no point in
-                // forking the async process off more than once so track that as well.  Use an archive lock to prevent more than
-                // one async process being launched.
-                if (!pushed && !forked &&
-                    lockAcquire(
-                        cfgOptionStr(cfgOptLockPath), cfgOptionStr(cfgOptStanza), cfgOptionStr(cfgOptExecId), cfgLockType(), 0,
-                        false))
+                // If the WAL segment has not already been pushed then start the async process to push it. There's no point in
+                // forking the async process off more than once so track that as well. Use an archive lock to prevent more than one
+                // async process being launched.
+                if (!pushed && !forked && lockAcquireP(.returnOnNoLock = true))
                 {
                     // The async process should not output on the console at all
                     KeyValue *optionReplace = kvNew();
@@ -374,7 +376,7 @@ cmdArchivePush(void)
                     // Execute the async process
                     archiveAsyncExec(archiveModePush, commandExec);
 
-                    // Mark the async process as forked so it doesn't get forked again.  A single run of the async process should be
+                    // Mark the async process as forked so it doesn't get forked again. A single run of the async process should be
                     // enough to do the job, running it again won't help anything.
                     forked = true;
                 }
@@ -388,8 +390,10 @@ cmdArchivePush(void)
             if (!pushed)
             {
                 THROW_FMT(
-                    ArchiveTimeoutError, "unable to push WAL file '%s' to the archive asynchronously after %s second(s)",
-                    strZ(archiveFile), strZ(cfgOptionDisplay(cfgOptArchiveTimeout)));
+                    ArchiveTimeoutError,
+                    "unable to push WAL file '%s' to the archive asynchronously after %s second(s)\n"
+                    "HINT: check '%s' for errors.",
+                    strZ(archiveFile), strZ(cfgOptionDisplay(cfgOptArchiveTimeout)), strZ(cfgLoadLogFileName(cfgCmdRoleAsync)));
             }
 
             // Log success
@@ -411,9 +415,9 @@ cmdArchivePush(void)
 
                 // Push the file to the archive
                 ArchivePushFileResult fileResult = archivePushFile(
-                    walFile, cfgOptionBool(cfgOptArchiveHeaderCheck), archiveInfo.pgVersion, archiveInfo.pgSystemId, archiveFile,
-                    compressTypeEnum(cfgOptionStrId(cfgOptCompressType)), cfgOptionInt(cfgOptCompressLevel), archiveInfo.repoList,
-                    archiveInfo.errorList);
+                    walFile, cfgOptionBool(cfgOptArchiveHeaderCheck), cfgOptionBool(cfgOptArchiveModeCheck), archiveInfo.pgVersion,
+                    archiveInfo.pgSystemId, archiveFile, compressTypeEnum(cfgOptionStrId(cfgOptCompressType)),
+                    cfgOptionInt(cfgOptCompressLevel), archiveInfo.repoList, archiveInfo.errorList);
 
                 // If a warning was returned then log it
                 for (unsigned int warnIdx = 0; warnIdx < strLstSize(fileResult.warnList); warnIdx++)
@@ -468,6 +472,7 @@ archivePushAsyncCallback(void *data, unsigned int clientIdx)
 
             pckWriteStrP(param, strNewFmt("%s/%s", strZ(jobData->walPath), strZ(walFile)));
             pckWriteBoolP(param, cfgOptionBool(cfgOptArchiveHeaderCheck));
+            pckWriteBoolP(param, cfgOptionBool(cfgOptArchiveModeCheck));
             pckWriteU32P(param, jobData->archiveInfo.pgVersion);
             pckWriteU64P(param, jobData->archiveInfo.pgSystemId);
             pckWriteStrP(param, walFile);
@@ -501,10 +506,10 @@ archivePushAsyncCallback(void *data, unsigned int clientIdx)
     }
     MEM_CONTEXT_TEMP_END();
 
-    FUNCTION_TEST_RETURN(result);
+    FUNCTION_TEST_RETURN(PROTOCOL_PARALLEL_JOB, result);
 }
 
-void
+FN_EXTERN void
 cmdArchivePushAsync(void)
 {
     FUNCTION_LOG_VOID(logLevelDebug);
@@ -544,7 +549,7 @@ cmdArchivePushAsync(void)
             LOG_INFO_FMT(
                 "push %u WAL file(s) to archive: %s%s", strLstSize(jobData.walFileList), strZ(strLstGet(jobData.walFileList, 0)),
                 strLstSize(jobData.walFileList) == 1 ?
-                    "" : strZ(strNewFmt("...%s", strZ(strLstGet(jobData.walFileList, strLstSize(jobData.walFileList) - 1)))));
+                    "" : zNewFmt("...%s", strZ(strLstGet(jobData.walFileList, strLstSize(jobData.walFileList) - 1))));
 
             // Drop files if queue max has been exceeded
             if (cfgOptionTest(cfgOptArchivePushQueueMax) && archivePushDrop(jobData.walPath, jobData.walFileList))
