@@ -7,6 +7,7 @@ Backup File
 #include "common/compress/helper.h"
 #include "common/crypto/common.h"
 #include "common/type/keyValue.h"
+#include "postgres/interface.h"
 
 /***********************************************************************************************************************************
 Backup file types
@@ -15,9 +16,9 @@ typedef enum
 {
     backupCopyResultChecksum,
     backupCopyResultCopy,
-    backupCopyResultReCopy,
     backupCopyResultSkip,
     backupCopyResultNoOp,
+    backupCopyResultTruncate,
 } BackupCopyResult;
 
 /***********************************************************************************************************************************
@@ -30,6 +31,7 @@ typedef struct BackupFile
     bool pgFileDelta;                                               // Checksum pg file before copying
     bool pgFileIgnoreMissing;                                       // Ignore missing pg file
     uint64_t pgFileSize;                                            // Expected pg file size
+    uint64_t pgFileSizePrior;                                       // Prior pg file size (if manifestFileHasReference)
     bool pgFileCopyExactSize;                                       // Copy only pg expected size
     const Buffer *pgFileChecksum;                                   // Expected pg file checksum
     bool pgFileChecksumPage;                                        // Validate page checksums?
@@ -51,9 +53,10 @@ typedef struct BackupFileResult
 {
     const String *manifestFile;                                     // Manifest file
     BackupCopyResult backupCopyResult;
+    bool repoInvalid;                                               // File was recopied because repo file was invalid
     uint64_t copySize;
-    Buffer *copyChecksum;
-    Buffer *repoChecksum;                                           // Checksum repo file (including compression, etc.)
+    const Buffer *copyChecksum;                                     // Checksum of pg file
+    const Buffer *repoChecksum;                                     // Checksum of repo file (including compression, etc.)
     uint64_t bundleOffset;                                          // Offset in bundle if any
     uint64_t repoSize;
     uint64_t blockIncrMapSize;                                      // Size of block incremental map (0 if no map)
@@ -62,6 +65,7 @@ typedef struct BackupFileResult
 
 FN_EXTERN List *backupFile(
     const String *repoFile, uint64_t bundleId, bool bundleRaw, unsigned int blockIncrReference, CompressType repoFileCompressType,
-    int repoFileCompressLevel, CipherType cipherType, const String *cipherPass, const String *pgVersionForce, const List *fileList);
+    int repoFileCompressLevel, CipherType cipherType, const String *cipherPass, const String *pgVersionForce, PgPageSize pageSize,
+    const List *fileList);
 
 #endif
