@@ -5,10 +5,10 @@ Server Command
 
 #include <sys/wait.h>
 
+#include "command/exit.h"
 #include "command/remote/remote.h"
 #include "command/server/server.h"
 #include "common/debug.h"
-#include "common/exit.h"
 #include "common/fork.h"
 #include "common/io/socket/server.h"
 #include "common/io/tls/server.h"
@@ -46,7 +46,7 @@ cmdServerInit(void)
     {
         MEM_CONTEXT_BEGIN(memContextTop())
         {
-            MEM_CONTEXT_NEW_BEGIN("Server")
+            MEM_CONTEXT_NEW_BEGIN(Server, .childQty = MEM_CONTEXT_QTY_MAX)
             {
                 serverLocal.memContext = MEM_CONTEXT_NEW();
                 serverLocal.processList = lstNewP(sizeof(pid_t));
@@ -98,18 +98,19 @@ cmdServerSigChild(const int signalType, siginfo_t *signalInfo, void *context)
     (void)signalType;
     (void)context;
 
-    ASSERT(signalInfo->si_code == CLD_EXITED);
-
     // Find the process and remove it
-    for (unsigned int processIdx  = 0; processIdx < lstSize(serverLocal.processList); processIdx++)
+    for (unsigned int processIdx = 0; processIdx < lstSize(serverLocal.processList); processIdx++)
     {
         if (*(int *)lstGet(serverLocal.processList, processIdx) == signalInfo->si_pid)
+        {
+            ASSERT(signalInfo->si_code == CLD_EXITED);
             lstRemoveIdx(serverLocal.processList, processIdx);
+        }
     }
 }
 
 /**********************************************************************************************************************************/
-void
+FN_EXTERN void
 cmdServer(const unsigned int argListSize, const char *argList[])
 {
     FUNCTION_LOG_BEGIN(logLevelDebug);
@@ -200,7 +201,7 @@ cmdServer(const unsigned int argListSize, const char *argList[])
     {
         sigaction(SIGCHLD, &(struct sigaction){.sa_flags = SA_NOCLDSTOP | SA_NOCLDWAIT}, NULL);
 
-        for (unsigned int processIdx  = 0; processIdx < lstSize(serverLocal.processList); processIdx++)
+        for (unsigned int processIdx = 0; processIdx < lstSize(serverLocal.processList); processIdx++)
         {
             pid_t pid = *(int *)lstGet(serverLocal.processList, processIdx);
 
