@@ -11,14 +11,14 @@ Execute Process
 #include <unistd.h>
 
 #include "common/debug.h"
-#include "common/log.h"
 #include "common/exec.h"
-#include "common/io/handleRead.h"
-#include "common/io/handleWrite.h"
+#include "common/fork.h"
+#include "common/io/fdRead.h"
+#include "common/io/fdWrite.h"
 #include "common/io/io.h"
-#include "common/io/read.intern.h"
-#include "common/io/write.intern.h"
-#include "common/type/object.h"
+#include "common/io/read.h"
+#include "common/io/write.h"
+#include "common/log.h"
 #include "common/wait.h"
 
 /***********************************************************************************************************************************
@@ -26,32 +26,26 @@ Object type
 ***********************************************************************************************************************************/
 struct Exec
 {
-    MemContext *memContext;                                         // Mem context
-    String *command;                                                // Command to execute
+    ExecPub pub;                                                    // Publicly accessible variables
     StringList *param;                                              // List of parameters to pass to command
     const String *name;                                             // Name to display in log/error messages
     TimeMSec timeout;                                               // Timeout for any i/o operation (read, write, etc.)
 
     pid_t processId;                                                // Process id of the child process
 
-    int handleRead;                                                 // Read handle
-    int handleWrite;                                                // Write handle
-    int handleError;                                                // Error handle
+    int fdRead;                                                     // Read file descriptor
+    int fdWrite;                                                    // Write file descriptor
+    int fdError;                                                    // Error file descriptor
 
-    IoRead *ioReadHandle;                                           // Handle read interface
-    IoWrite *ioWriteHandle;                                         // Handle write interface
-
-    IoRead *ioReadExec;                                             // Wrapper for handle read interface
-    IoWrite *ioWriteExec;                                           // Wrapper for handle write interface
+    IoRead *ioReadFd;                                               // File descriptor read interface
+    IoWrite *ioWriteFd;                                             // File descriptor write interface
 };
 
-OBJECT_DEFINE_FREE(EXEC);
-
 /***********************************************************************************************************************************
 Macro to close file descriptors after dup2() in the child process
 
 If the parent process is daemomized and has closed stdout, stdin, and stderr or some combination of them, then the newly created
-descriptors might overlap stdout, stdin, or stderr.  In that case we don't want to accidentally close the descriptor that we have
+descriptors might overlap stdout, stdin, or stderr. In that case we don't want to accidentally close the descriptor that we have
 just copied.
 
 Note that this is pretty specific to the way that file descriptors are handled in this module and may not be generally applicable in
@@ -71,14 +65,23 @@ other code.
     while (0);
 
 /***********************************************************************************************************************************
-Free exec handles and ensure process is shut down
+Free exec file descriptors and ensure process is shut down
 ***********************************************************************************************************************************/
-OBJECT_DEFINE_FREE_RESOURCE_BEGIN(EXEC, LOG, logLevelTrace)
+static void
+execFreeResource(THIS_VOID)
 {
-    // Close the io handles
-    close(this->handleRead);
-    close(this->handleWrite);
-    close(this->handleError);
+    THIS(Exec);
+
+    FUNCTION_LOG_BEGIN(logLevelTrace);
+        FUNCTION_LOG_PARAM(EXEC, this);
+    FUNCTION_LOG_END();
+
+    ASSERT(this != NULL);
+
+    // Close file descriptors
+    close(this->fdRead);
+    close(this->fdWrite);
+    close(this->fdError);
 
     // Wait for the child to exit. We don't really care how it exits as long as it does.
     if (this->processId != 0)
@@ -98,18 +101,19 @@ OBJECT_DEFINE_FREE_RESOURCE_BEGIN(EXEC, LOG, logLevelTrace)
 
             // If the process did not exit then error -- else we may end up with a collection of zombie processes
             if (processResult == 0)
-                THROW_FMT(ExecuteError, "%s did not exit when expected", strPtr(this->name));
+                THROW_FMT(ExecuteError, "%s did not exit when expected", strZ(this->name));
         }
         MEM_CONTEXT_TEMP_END();
     }
+
+    FUNCTION_LOG_RETURN_VOID();
 }
-OBJECT_DEFINE_FREE_RESOURCE_END(LOG);
 
 /**********************************************************************************************************************************/
-Exec *
+FN_EXTERN Exec *
 execNew(const String *command, const StringList *param, const String *name, TimeMSec timeout)
 {
-    FUNCTION_LOG_BEGIN(logLevelDebug)
+    FUNCTION_LOG_BEGIN(logLevelDebug);
         FUNCTION_LOG_PARAM(STRING, command);
         FUNCTION_LOG_PARAM(STRING_LIST, param);
         FUNCTION_LOG_PARAM(STRING, name);
@@ -120,16 +124,14 @@ execNew(const String *command, const StringList *param, const String *name, Time
     ASSERT(name != NULL);
     ASSERT(timeout > 0);
 
-    Exec *this = NULL;
-
-    MEM_CONTEXT_NEW_BEGIN("Exec")
+    OBJ_NEW_BEGIN(Exec, .childQty = MEM_CONTEXT_QTY_MAX, .callbackQty = 1)
     {
-        this = memNew(sizeof(Exec));
-
         *this = (Exec)
         {
-            .memContext = MEM_CONTEXT_NEW(),
-            .command = strDup(command),
+            .pub =
+            {
+                .command = strDup(command),
+            },
             .name = strDup(name),
             .timeout = timeout,
 
@@ -138,9 +140,9 @@ execNew(const String *command, const StringList *param, const String *name, Time
         };
 
         // The first parameter must be the command
-        strLstInsert(this->param, 0, this->command);
+        strLstInsert(this->param, 0, execCommand(this));
     }
-    MEM_CONTEXT_NEW_END();
+    OBJ_NEW_END();
 
     FUNCTION_LOG_RETURN(EXEC, this);
 }
@@ -148,9 +150,34 @@ execNew(const String *command, const StringList *param, const String *name, Time
 /***********************************************************************************************************************************
 Check if the process is still running
 
-This should be called when anything unexpected happens while reading or writing, including errors and eof.  If this function returns
+This should be called when anything unexpected happens while reading or writing, including errors and eof. If this function returns
 then the original error should be rethrown.
 ***********************************************************************************************************************************/
+// Helper to throw status error
+static void
+execCheckStatusError(Exec *const this, const int status, const String *const output)
+{
+    FUNCTION_TEST_BEGIN();
+        FUNCTION_LOG_PARAM(EXEC, this);
+        FUNCTION_TEST_PARAM(INT, status);
+        FUNCTION_TEST_PARAM(STRING, output);
+    FUNCTION_TEST_END();
+
+    // Throw the error with as much information as is available
+    THROWP_FMT(
+        errorTypeFromCode(WEXITSTATUS(status)), "%s terminated unexpectedly [%d]%s%s", strZ(this->name),
+        WEXITSTATUS(status), strSize(output) > 0 ? ": " : "", strSize(output) > 0 ? strZ(output) : "");
+
+    FUNCTION_TEST_RETURN_VOID();
+}
+
+// Helper to throw signal error
+static void
+execCheckSignalError(Exec *const this, const int status)
+{
+    THROW_FMT(ExecuteError, "%s terminated unexpectedly on signal %d", strZ(this->name), WTERMSIG(status));
+}
+
 static void
 execCheck(Exec *this)
 {
@@ -174,19 +201,13 @@ execCheck(Exec *this)
         // If the process exited normally
         if (WIFEXITED(processStatus))
         {
-            // Get data from stderr to help diagnose the problem
-            IoRead *ioReadError = ioHandleReadNew(strNewFmt("%s error", strPtr(this->name)), this->handleError, 0);
-            ioReadOpen(ioReadError);
-            String *errorStr = strTrim(strNewBuf(ioReadBuf(ioReadError)));
-
-            // Throw the error with as much information as is available
-            THROWP_FMT(
-                errorTypeFromCode(WEXITSTATUS(processStatus)), "%s terminated unexpectedly [%d]%s%s", strPtr(this->name),
-                WEXITSTATUS(processStatus), strSize(errorStr) > 0 ? ": " : "", strSize(errorStr) > 0 ? strPtr(errorStr) : "");
+            execCheckStatusError(
+                this, processStatus,
+                strTrim(strNewBuf(ioReadBuf(ioFdReadNewOpen(strNewFmt("%s error", strZ(this->name)), this->fdError, 0)))));
         }
 
         // If the process did not exit normally then it must have been a signal
-        THROW_FMT(ExecuteError, "%s terminated unexpectedly on signal %d", strPtr(this->name), WTERMSIG(processStatus));
+        execCheckSignalError(this, processStatus);
     }
 
     FUNCTION_LOG_RETURN_VOID();
@@ -213,7 +234,7 @@ execRead(THIS_VOID, Buffer *buffer, bool block)
 
     TRY_BEGIN()
     {
-        result = ioReadInterface(this->ioReadHandle)->read(ioReadDriver(this->ioReadHandle), buffer, block);
+        result = ioReadInterface(this->ioReadFd)->read(ioReadDriver(this->ioReadFd), buffer, block);
     }
     CATCH_ANY()
     {
@@ -243,8 +264,8 @@ execWrite(THIS_VOID, const Buffer *buffer)
 
     TRY_BEGIN()
     {
-        ioWrite(this->ioWriteHandle, buffer);
-        ioWriteFlush(this->ioWriteHandle);
+        ioWrite(this->ioWriteFd, buffer);
+        ioWriteFlush(this->ioWriteFd);
     }
     CATCH_ANY()
     {
@@ -271,17 +292,17 @@ execEof(THIS_VOID)
     ASSERT(this != NULL);
 
     // Check if the process is still running on eof
-    if (ioReadInterface(this->ioReadHandle)->eof(ioReadDriver(this->ioReadHandle)))
+    if (ioReadInterface(this->ioReadFd)->eof(ioReadDriver(this->ioReadFd)))
         execCheck(this);
 
     FUNCTION_LOG_RETURN(BOOL, false);
 }
 
 /***********************************************************************************************************************************
-Get the read handle
+Get the read file descriptor
 ***********************************************************************************************************************************/
 static int
-execHandleRead(const THIS_VOID)
+execFdRead(const THIS_VOID)
 {
     THIS(const Exec);
 
@@ -291,20 +312,20 @@ execHandleRead(const THIS_VOID)
 
     ASSERT(this != NULL);
 
-    FUNCTION_TEST_RETURN(this->handleRead);
+    FUNCTION_TEST_RETURN(INT, this->fdRead);
 }
 
 /**********************************************************************************************************************************/
-void
+FN_EXTERN void
 execOpen(Exec *this)
 {
-    FUNCTION_LOG_BEGIN(logLevelDebug)
+    FUNCTION_LOG_BEGIN(logLevelDebug);
         FUNCTION_LOG_PARAM(EXEC, this);
     FUNCTION_LOG_END();
 
     ASSERT(this != NULL);
 
-    // Create pipes to communicate with the subprocess.  The names of the pipes are from the perspective of the parent process since
+    // Create pipes to communicate with the subprocess. The names of the pipes are from the perspective of the parent process since
     // the child process will use them only briefly before exec'ing.
     int pipeRead[2];
     int pipeWrite[2];
@@ -315,7 +336,7 @@ execOpen(Exec *this)
     THROW_ON_SYS_ERROR(pipe(pipeError) == -1, KernelError, "unable to create error pipe");
 
     // Fork the subprocess
-    this->processId = fork();
+    this->processId = forkSafe();
 
     // Exec command in the child process
     if (this->processId == 0)
@@ -332,77 +353,41 @@ execOpen(Exec *this)
         // Assign stderr to the input side of the error pipe
         PIPE_DUP2(pipeError, 1, STDERR_FILENO);
 
-        // Execute the binary.  This statement will not return if it is successful
-        execvp(strPtr(this->command), (char ** const)strLstPtr(this->param));
+        // Execute the binary. This statement will not return if it is successful
+        execvp(strZ(execCommand(this)), (char **const)strLstPtr(this->param));
 
-        // If we got here then there was an error.  We can't use a throw as we normally would because we have already shutdown
+        // If we got here then there was an error. We can't use a throw as we normally would because we have already shutdown
         // logging and we don't want to execute exit paths that might free parent resources which we still have references to.
-        fprintf(stderr, "unable to execute '%s': [%d] %s\n", strPtr(this->command), errno, strerror(errno));
+        fprintf(stderr, "unable to execute '%s': [%d] %s\n", strZ(execCommand(this)), errno, strerror(errno));
         exit(errorTypeCode(&ExecuteError));
     }
 
-    // Close the unused handles
+    // Close the unused file descriptors
     close(pipeRead[1]);
     close(pipeWrite[0]);
     close(pipeError[1]);
 
-    // Store the handles we'll use and need to close when the process terminates
-    this->handleRead = pipeRead[0];
-    this->handleWrite = pipeWrite[1];
-    this->handleError = pipeError[0];
+    // Store the file descriptors we'll use and need to close when the process terminates
+    this->fdRead = pipeRead[0];
+    this->fdWrite = pipeWrite[1];
+    this->fdError = pipeError[0];
 
-    // Assign handles to io interfaces
-    this->ioReadHandle = ioHandleReadNew(strNewFmt("%s read", strPtr(this->name)), this->handleRead, this->timeout);
-    this->ioWriteHandle = ioHandleWriteNew(strNewFmt("%s write", strPtr(this->name)), this->handleWrite);
-    ioWriteOpen(this->ioWriteHandle);
+    MEM_CONTEXT_OBJ_BEGIN(this)
+    {
+        // Assign file descriptors to io interfaces
+        this->ioReadFd = ioFdReadNew(strNewFmt("%s read", strZ(this->name)), this->fdRead, this->timeout);
+        this->ioWriteFd = ioFdWriteNewOpen(strNewFmt("%s write", strZ(this->name)), this->fdWrite, this->timeout);
 
-    // Create wrapper interfaces that check process state
-    this->ioReadExec = ioReadNewP(this, .block = true, .read = execRead, .eof = execEof, .handle = execHandleRead);
-    ioReadOpen(this->ioReadExec);
-    this->ioWriteExec = ioWriteNewP(this, .write = execWrite);
-    ioWriteOpen(this->ioWriteExec);
+        // Create wrapper interfaces that check process state
+        this->pub.ioReadExec = ioReadNewP(this, .block = true, .read = execRead, .eof = execEof, .fd = execFdRead);
+        ioReadOpen(execIoRead(this));
+        this->pub.ioWriteExec = ioWriteNewP(this, .write = execWrite);
+        ioWriteOpen(execIoWrite(this));
+    }
+    MEM_CONTEXT_OBJ_END();
 
-    // Set a callback so the handles will get freed
-    memContextCallbackSet(this->memContext, execFreeResource, this);
+    // Set a callback so the file descriptors will get freed
+    memContextCallbackSet(objMemContext(this), execFreeResource, this);
 
     FUNCTION_LOG_RETURN_VOID();
 }
-
-/**********************************************************************************************************************************/
-IoRead *
-execIoRead(const Exec *this)
-{
-    FUNCTION_TEST_BEGIN();
-        FUNCTION_TEST_PARAM(EXEC, this);
-    FUNCTION_TEST_END();
-
-    ASSERT(this != NULL);
-
-    FUNCTION_TEST_RETURN(this->ioReadExec);
-}
-
-/**********************************************************************************************************************************/
-IoWrite *
-execIoWrite(const Exec *this)
-{
-    FUNCTION_TEST_BEGIN();
-        FUNCTION_TEST_PARAM(EXEC, this);
-    FUNCTION_TEST_END();
-
-    ASSERT(this != NULL);
-
-    FUNCTION_TEST_RETURN(this->ioWriteExec);
-}
-
-/**********************************************************************************************************************************/
-MemContext *
-execMemContext(const Exec *this)
-{
-    FUNCTION_TEST_BEGIN();
-        FUNCTION_TEST_PARAM(EXEC, this);
-    FUNCTION_TEST_END();
-
-    ASSERT(this != NULL);
-
-    FUNCTION_TEST_RETURN(this->memContext);
-}
