@@ -67,6 +67,15 @@ bool testContainer(void);
 // parallel.
 unsigned int testIdx(void);
 
+// PostgreSQL version for integration testing
+const char *testPgVersion(void);
+
+// User running the test
+const char *testUser(void);
+
+// VM for integration testing
+const char *testVm(void);
+
 /***********************************************************************************************************************************
 Test that an expected error is actually thrown and error when it isn't
 ***********************************************************************************************************************************/
@@ -142,6 +151,14 @@ Output information about the test
     puts(comment);                                                                                                                 \
     fflush(stdout);
 
+#define TEST_RESULT_INFO_LINE_FMT(line, comment, ...)                                                                              \
+    hrnTestLogPrefix(line);                                                                                                        \
+    printf(comment "\n", __VA_ARGS__);                                                                                             \
+    fflush(stdout)
+
+#define TEST_RESULT_INFO_FMT(comment, ...)                                                                                         \
+    TEST_RESULT_INFO_LINE_FMT(__LINE__, comment, __VA_ARGS__)
+
 /***********************************************************************************************************************************
 Test that a void statement returns and does not throw an error
 ***********************************************************************************************************************************/
@@ -323,6 +340,20 @@ Test log result
         TRY_END();                                                                                                                 \
     } while (0)
 
+#define TEST_RESULT_LOG_EMPTY_OR_CONTAINS(expected)                                                                                \
+    do                                                                                                                             \
+    {                                                                                                                              \
+        TRY_BEGIN()                                                                                                                \
+        {                                                                                                                          \
+            harnessLogResultEmptyOrContains(expected);                                                                             \
+        }                                                                                                                          \
+        CATCH_ANY()                                                                                                                \
+        {                                                                                                                          \
+            THROW_FMT(AssertError, "LOG RESULT CONTAINS FAILED WITH:\n%s", errorMessage());                                        \
+        }                                                                                                                          \
+        TRY_END();                                                                                                                 \
+    } while (0)
+
 #define TEST_RESULT_LOG_FMT(...)                                                                                                   \
     do                                                                                                                             \
     {                                                                                                                              \
