@@ -90,6 +90,9 @@ testRun(void)
 
         TEST_RESULT_BOOL(
             archivePushDrop(STRDEF("pg_wal"), archivePushProcessList(STRDEF(TEST_PATH "/db/pg_wal"))), true, "wal is dropped");
+
+        // No WAL to be processed
+        TEST_RESULT_BOOL(archivePushDrop(STRDEF("pg_wal"), strLstNew()), false, "no WAL to be processed");
     }
 
     // *****************************************************************************************************************************
@@ -469,7 +472,7 @@ testRun(void)
             "unexpected WAL magic 999\n"
             "HINT: is this version of PostgreSQL supported?");
 
-        HRN_PG_CONTROL_OVERRIDE_PUT(storagePgWrite(), PG_VERSION_11, 1501, .catalogVersion = 202211111);
+        HRN_PG_CONTROL_OVERRIDE_VERSION_PUT(storagePgWrite(), PG_VERSION_11, 1501, .catalogVersion = 202211111);
 
         TEST_ERROR(
             cmdArchivePush(), VersionNotSupportedError,
