@@ -19,7 +19,12 @@ storageTestPathExpression(const String *expression, const String *path)
     String *result = NULL;
 
     if (strcmp(strZ(expression), "<TEST>") == 0)
-        result = strNewFmt("test%s", path == NULL ? "" : strZ(strNewFmt("/%s", strZ(path))));
+    {
+        result = strCatZ(strNew(), "test");
+
+        if (path != NULL)
+            strCatFmt(result, "/%s", strZ(path));
+    }
     else if (strcmp(strZ(expression), "<NULL>") != 0)
         THROW_FMT(AssertError, "invalid expression '%s'", strZ(expression));
 
@@ -202,7 +207,7 @@ testRun(void)
         TEST_RESULT_BOOL(storageInfoP(storageRootNoPath, NULL, .ignoreMissing = true).exists, false, "no info for /");
 
         // -------------------------------------------------------------------------------------------------------------------------
-        TEST_TITLE("directory does not exists");
+        TEST_TITLE("directory does not exist");
 
         const String *fileName = STRDEF(TEST_PATH "/fileinfo");
 
@@ -211,7 +216,7 @@ testRun(void)
             strZ(fileName));
 
         // -------------------------------------------------------------------------------------------------------------------------
-        TEST_TITLE("file does not exists");
+        TEST_TITLE("file does not exist");
 
         StorageInfo info = {0};
         TEST_ASSIGN(info, storageInfoP(storageTest, fileName, .ignoreMissing = true), "get file info (missing)");
@@ -340,7 +345,7 @@ testRun(void)
     }
 
     // *****************************************************************************************************************************
-    if (testBegin("storageInfoList()"))
+    if (testBegin("storageNewItrP()"))
     {
 #ifdef TEST_CONTAINER_REQUIRED
         TEST_CREATE_NOPERM();
@@ -350,49 +355,43 @@ testRun(void)
         TEST_TITLE("path missing");
 
         TEST_ERROR_FMT(
-            storageInfoListP(storageTest, STRDEF(BOGUS_STR), (StorageInfoListCallback)1, NULL, .errorOnMissing = true),
-            PathMissingError, STORAGE_ERROR_LIST_INFO_MISSING, TEST_PATH "/BOGUS");
+            storageNewItrP(storageTest, STRDEF(BOGUS_STR), .errorOnMissing = true), PathMissingError,
+            STORAGE_ERROR_LIST_INFO_MISSING, TEST_PATH "/BOGUS");
 
-        TEST_RESULT_BOOL(
-            storageInfoListP(storageTest, STRDEF(BOGUS_STR), (StorageInfoListCallback)1, NULL), false, "ignore missing dir");
+        TEST_RESULT_PTR(storageNewItrP(storageTest, STRDEF(BOGUS_STR), .nullOnMissing = true), NULL, "ignore missing dir");
 
 #ifdef TEST_CONTAINER_REQUIRED
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("error on permission denied, regardless of errorOnMissing setting");
+
         TEST_ERROR_FMT(
-            storageInfoListP(storageTest, pathNoPerm, (StorageInfoListCallback)1, NULL), PathOpenError,
+            storageNewItrP(storageTest, pathNoPerm, .errorOnMissing = true), PathOpenError,
             STORAGE_ERROR_LIST_INFO ": [13] Permission denied", strZ(pathNoPerm));
 
         // Should still error even when ignore missing
         TEST_ERROR_FMT(
-            storageInfoListP(storageTest, pathNoPerm, (StorageInfoListCallback)1, NULL), PathOpenError,
-            STORAGE_ERROR_LIST_INFO ": [13] Permission denied", strZ(pathNoPerm));
+            storageNewItrP(storageTest, pathNoPerm), PathOpenError, STORAGE_ERROR_LIST_INFO ": [13] Permission denied",
+            strZ(pathNoPerm));
 #endif // TEST_CONTAINER_REQUIRED
 
         // -------------------------------------------------------------------------------------------------------------------------
-        TEST_TITLE("helper function - storagePosixInfoListEntry()");
-
-        HarnessStorageInfoListCallbackData callbackData =
-        {
-            .content = strNew(),
-        };
+        TEST_TITLE("helper function - storagePosixListEntry()");
 
         TEST_RESULT_VOID(
-            storagePosixInfoListEntry(
-                (StoragePosix *)storageDriver(storageTest), STRDEF("pg"), STRDEF("missing"), storageInfoLevelBasic,
-                hrnStorageInfoListCallback, &callbackData),
+            storagePosixListEntry(
+                (StoragePosix *)storageDriver(storageTest), storageLstNew(storageInfoLevelBasic), STRDEF("pg"), "missing",
+                storageInfoLevelBasic),
             "missing path");
-        TEST_RESULT_STR_Z(callbackData.content, "", "check content");
 
         // -------------------------------------------------------------------------------------------------------------------------
         TEST_TITLE("path with only dot");
 
         storagePathCreateP(storageTest, STRDEF("pg"), .mode = 0766);
 
-        callbackData.content = strNew();
-
-        TEST_RESULT_VOID(
-            storageInfoListP(storageTest, STRDEF("pg"), hrnStorageInfoListCallback, &callbackData),
-            "directory with one dot file sorted");
-        TEST_RESULT_STR_Z(callbackData.content, ". {path, m=0766, u=" TEST_USER ", g=" TEST_GROUP "}\n", "check content");
+        TEST_STORAGE_LIST(
+            storageTest, "pg",
+            "./ {u=" TEST_USER ", g=" TEST_GROUP ", m=0766}\n",
+            .level = storageInfoLevelDetail, .includeDot = true);
 
         // -------------------------------------------------------------------------------------------------------------------------
         TEST_TITLE("path with file, link, pipe");
@@ -402,114 +401,125 @@ testRun(void)
         HRN_SYSTEM("sudo chown 77777:77777 " TEST_PATH "/pg/.include");
 #endif // TEST_CONTAINER_REQUIRED
 
-        storagePutP(storageNewWriteP(storageTest, STRDEF("pg/file"), .modeFile = 0660), BUFSTRDEF("TESTDATA"));
+        storagePutP(
+            storageNewWriteP(storageTest, STRDEF("pg/file"), .modeFile = 0660, .timeModified = 1656433838), BUFSTRDEF("TESTDATA"));
 
         HRN_SYSTEM("ln -s ../file " TEST_PATH "/pg/link");
         HRN_SYSTEM("mkfifo -m 777 " TEST_PATH "/pg/pipe");
 
-        callbackData = (HarnessStorageInfoListCallbackData)
-        {
-            .content = strNew(),
-            .timestampOmit = true,
-            .modeOmit = true,
-            .modePath = 0766,
-            .modeFile = 0600,
-            .userOmit = true,
-            .groupOmit = true,
-        };
-
-        TEST_RESULT_VOID(
-            storageInfoListP(storageTest, STRDEF("pg"), hrnStorageInfoListCallback, &callbackData, .sortOrder = sortOrderAsc),
-            "directory with one dot file sorted");
-        TEST_RESULT_STR_Z(
-            callbackData.content,
-            ". {path}\n"
+        TEST_STORAGE_LIST(
+            storageTest, "pg",
+            "./ {u=" TEST_USER ", g=" TEST_GROUP ", m=0766}\n"
 #ifdef TEST_CONTAINER_REQUIRED
-            ".include {path, m=0755, u=77777, g=77777}\n"
+            ".include/ {u=77777, g=77777, m=0755}\n"
 #endif // TEST_CONTAINER_REQUIRED
-            "file {file, s=8, m=0660}\n"
-            "link {link, d=../file}\n"
-            "pipe {special}\n",
-            "check content");
+            "file {s=8, t=1656433838, u=" TEST_USER ", g=" TEST_GROUP ", m=0660}\n"
+            "link> {d=../file, u=" TEST_USER ", g=" TEST_GROUP "}\n"
+            "pipe*\n",
+            .level = storageInfoLevelDetail, .includeDot = true);
 
 #ifdef TEST_CONTAINER_REQUIRED
         HRN_SYSTEM("sudo rmdir " TEST_PATH "/pg/.include");
 #endif // TEST_CONTAINER_REQUIRED
 
         // -------------------------------------------------------------------------------------------------------------------------
-        TEST_TITLE("path - recurse");
+        TEST_TITLE("storageItrMore() twice in a row");
+
+        StorageIterator *storageItr = NULL;
+
+        TEST_ASSIGN(storageItr, storageNewItrP(storageTest, STRDEF("pg")), "new iterator");
+        TEST_RESULT_BOOL(storageItrMore(storageItr), true, "check more");
+        TEST_RESULT_BOOL(storageItrMore(storageItr), true, "check more again");
+
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("path - recurse desc");
 
         storagePathCreateP(storageTest, STRDEF("pg/path"), .mode = 0700);
-        storagePutP(storageNewWriteP(storageTest, STRDEF("pg/path/file"), .modeFile = 0600), BUFSTRDEF("TESTDATA"));
+        storagePutP(
+            storageNewWriteP(storageTest, STRDEF("pg/path/file"), .modeFile = 0600, .timeModified = 1656434296),
+            BUFSTRDEF("TESTDATA"));
 
-        callbackData.content = strNew();
+        TEST_STORAGE_LIST(
+            storageTest, "pg",
+            "pipe*\n"
+            "path/file {s=8, t=1656434296}\n"
+            "path/\n"
+            "link> {d=../file}\n"
+            "file {s=8, t=1656433838}\n"
+            "./\n",
+            .level = storageInfoLevelBasic, .includeDot = true, .sortOrder = sortOrderDesc);
 
-        TEST_RESULT_VOID(
-            storageInfoListP(
-                storageTest, STRDEF("pg"), hrnStorageInfoListCallback, &callbackData, .sortOrder = sortOrderDesc, .recurse = true),
-            "recurse descending");
-        TEST_RESULT_STR_Z(
-            callbackData.content,
-            "pipe {special}\n"
-            "path/file {file, s=8}\n"
-            "path {path, m=0700}\n"
-            "link {link, d=../file}\n"
-            "file {file, s=8, m=0660}\n"
-            ". {path}\n",
-            "check content");
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("path - recurse asc");
+
+        // Create a path with a subpath that will always be last to make sure lists are not freed too early in the iterator
+        storagePathCreateP(storageTest, STRDEF("pg/zzz/yyy"), .mode = 0700);
+
+        TEST_STORAGE_LIST(
+            storageTest, "pg",
+            "./\n"
+            "file {s=8, t=1656433838}\n"
+            "link> {d=../file}\n"
+            "path/\n"
+            "path/file {s=8, t=1656434296}\n"
+            "pipe*\n"
+            "zzz/\n"
+            "zzz/yyy/\n",
+            .level = storageInfoLevelBasic, .includeDot = true);
 
         // -------------------------------------------------------------------------------------------------------------------------
         TEST_TITLE("path basic info - recurse");
 
-        storagePathCreateP(storageTest, STRDEF("pg/path"), .mode = 0700);
-        storagePutP(storageNewWriteP(storageTest, STRDEF("pg/path/file"), .modeFile = 0600), BUFSTRDEF("TESTDATA"));
-
-        callbackData.content = strNew();
-
         storageTest->pub.interface.feature ^= 1 << storageFeatureInfoDetail;
 
-        TEST_RESULT_VOID(
-            storageInfoListP(
-                storageTest, STRDEF("pg"), hrnStorageInfoListCallback, &callbackData, .sortOrder = sortOrderDesc, .recurse = true),
-            "recurse descending");
-        TEST_RESULT_STR_Z(
-            callbackData.content,
-            "pipe {special}\n"
-            "path/file {file, s=8}\n"
-            "path {path}\n"
-            "link {link}\n"
-            "file {file, s=8}\n"
-            ". {path}\n",
-            "check content");
+        TEST_STORAGE_LIST(
+            storageTest, "pg",
+            "zzz/yyy/\n"
+            "zzz/\n"
+            "pipe*\n"
+            "path/file {s=8, t=1656434296}\n"
+            "path/\n"
+            "link> {d=../file}\n"
+            "file {s=8, t=1656433838}\n"
+            "./\n",
+            .levelForce = true, .includeDot = true, .sortOrder = sortOrderDesc);
 
         storageTest->pub.interface.feature ^= 1 << storageFeatureInfoDetail;
 
         // -------------------------------------------------------------------------------------------------------------------------
-        TEST_TITLE("path - filter");
+        TEST_TITLE("path basic info - no recurse");
 
-        callbackData.content = strNew();
+        storageTest->pub.interface.feature ^= 1 << storageFeatureInfoDetail;
 
-        TEST_RESULT_VOID(
-            storageInfoListP(
-                storageTest, STRDEF("pg"), hrnStorageInfoListCallback, &callbackData, .sortOrder = sortOrderAsc,
-                .expression = STRDEF("^path")),
-            "filter");
-        TEST_RESULT_STR_Z(
-            callbackData.content,
-            "path {path, m=0700}\n",
-            "check content");
+        TEST_STORAGE_LIST(
+            storageTest, "pg",
+            "zzz/\n"
+            "pipe*\n"
+            "path/\n"
+            "link> {d=../file}\n"
+            "file {s=8, t=1656433838}\n"
+            "./\n",
+            .levelForce = true, .includeDot = true, .noRecurse = true, .sortOrder = sortOrderDesc);
+
+        storageTest->pub.interface.feature ^= 1 << storageFeatureInfoDetail;
+
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("empty path - filter");
+
+        storagePathCreateP(storageTest, STRDEF("pg/empty"), .mode = 0700);
+
+        TEST_STORAGE_LIST(
+            storageTest, "pg",
+            "empty/\n",
+            .level = storageInfoLevelType, .expression = "^empty");
 
         // -------------------------------------------------------------------------------------------------------------------------
         TEST_TITLE("filter in subpath during recursion");
 
-        callbackData.content = strNew();
-
-        TEST_RESULT_VOID(
-            storageInfoListP(
-                storageTest, STRDEF("pg"), hrnStorageInfoListCallback, &callbackData, .sortOrder = sortOrderAsc, .recurse = true,
-                .expression = STRDEF("\\/file$")),
-            "filter");
-        TEST_RESULT_STR_Z(callbackData.content, "path/file {file, s=8}\n", "check content");
+        TEST_STORAGE_LIST(
+            storageTest, "pg",
+            "path/file {s=8, t=1656434296}\n",
+            .level = storageInfoLevelBasic, .expression = "\\/file$");
     }
 
     // *****************************************************************************************************************************
@@ -531,7 +541,7 @@ testRun(void)
 
 #ifdef TEST_CONTAINER_REQUIRED
         // -------------------------------------------------------------------------------------------------------------------------
-        TEST_TITLE("error on missing, regardless of errorOnMissing setting");
+        TEST_TITLE("error on permission denied, regardless of errorOnMissing setting");
 
         TEST_ERROR_FMT(
             storageListP(storageTest, pathNoPerm, .errorOnMissing = true), PathOpenError,
@@ -539,16 +549,17 @@ testRun(void)
 
         // Should still error even when ignore missing
         TEST_ERROR_FMT(
-            storageListP(storageTest, pathNoPerm), PathOpenError,
-            STORAGE_ERROR_LIST_INFO ": [13] Permission denied", strZ(pathNoPerm));
+            storageListP(storageTest, pathNoPerm), PathOpenError, STORAGE_ERROR_LIST_INFO ": [13] Permission denied",
+            strZ(pathNoPerm));
 #endif // TEST_CONTAINER_REQUIRED
 
         // -------------------------------------------------------------------------------------------------------------------------
         TEST_TITLE("list - path with files");
 
-        TEST_RESULT_VOID(
-            storagePutP(storageNewWriteP(storageTest, STRDEF(".aaa.txt")), BUFSTRDEF("aaa")), "write aaa.text");
-        TEST_RESULT_STRLST_Z(strLstSort(storageListP(storageTest, NULL), sortOrderAsc), ".aaa.txt\n"
+        TEST_RESULT_VOID(storagePutP(storageNewWriteP(storageTest, STRDEF(".aaa.txt")), BUFSTRDEF("aaa")), "write aaa.text");
+        TEST_RESULT_STRLST_Z(
+            strLstSort(storageListP(storageTest, NULL), sortOrderAsc),
+            ".aaa.txt\n"
 #ifdef TEST_CONTAINER_REQUIRED
             "noperm\n"
 #endif // TEST_CONTAINER_REQUIRED
@@ -875,6 +886,50 @@ testRun(void)
         TEST_RESULT_VOID(storagePathSyncP(storageTest, pathName), "sync path");
     }
 
+    // *****************************************************************************************************************************
+    if (testBegin("storageLinkCreate()"))
+    {
+        StorageInfo info = {0};
+        const String *backupLabel = STRDEF("20181119-152138F");
+        const String *latestLabel = STRDEF("latest");
+        int invalidLinkType = 9;
+
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("create soft link to BACKUPLABEL");
+
+        TEST_RESULT_VOID(storagePathCreateP(storageTest, backupLabel), "create path to link to");
+        TEST_RESULT_VOID(storageLinkCreateP(storageTest, backupLabel, latestLabel), "create symlink");
+        TEST_ASSIGN(info, storageInfoP(storageTest, latestLabel, .ignoreMissing = false), "get link info");
+        TEST_RESULT_STR(info.linkDestination, backupLabel, "match link destination");
+        TEST_RESULT_VOID(storageRemoveP(storageTest, latestLabel), "remove symlink");
+
+        TEST_RESULT_VOID(storageLinkCreateP(storageTest, backupLabel, latestLabel, .linkType = storageLinkSym), "create symlink");
+        TEST_ASSIGN(info, storageInfoP(storageTest, latestLabel, .ignoreMissing = false), "get link info");
+        TEST_RESULT_STR(info.linkDestination, backupLabel, "match link destination");
+        TEST_RESULT_VOID(storageRemoveP(storageTest, latestLabel), "remove symlink");
+
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("hardlink success/fail");
+
+        const String *emptyFile = STRDEF(TEST_PATH "/test.empty");
+        TEST_RESULT_VOID(storagePutP(storageNewWriteP(storageTest, emptyFile), NULL), "put empty file");
+        TEST_RESULT_VOID(
+            storageLinkCreateP(storageTest, emptyFile, latestLabel, .linkType = storageLinkHard), "hardlink to empty file");
+        TEST_RESULT_VOID(storageRemoveP(storageTest, emptyFile), "remove empty file");
+        TEST_RESULT_VOID(storageRemoveP(storageTest, latestLabel), "remove latest label");
+
+        TEST_ERROR(
+            storageLinkCreateP(storageTest, backupLabel, latestLabel, .linkType = storageLinkHard), FileOpenError,
+            "unable to create hardlink \'latest\' to \'20181119-152138F\': [1] Operation not permitted");
+        TEST_ASSIGN(info, storageInfoP(storageTest, latestLabel, .ignoreMissing = true), "get link info");
+        TEST_RESULT_STR(info.linkDestination, NULL, "no link destination");
+        TEST_RESULT_VOID(storagePathRemoveP(storageTest, backupLabel), "remove backup path");
+        TEST_ERROR(
+            storageLinkCreateP(storageTest, backupLabel, latestLabel, .linkType = invalidLinkType), AssertError,
+            "assertion '(param.linkType == storageLinkSym && storageFeature(this, storageFeatureSymLink)) ||"
+            " (param.linkType == storageLinkHard && storageFeature(this, storageFeatureHardLink))' failed");
+    }
+
     // *****************************************************************************************************************************
     if (testBegin("storageNewRead()"))
     {
@@ -935,6 +990,12 @@ testRun(void)
         fileName = STRDEF(TEST_PATH "/sub1/testfile-abort");
         String *fileNameTmp = strNewFmt("%s." STORAGE_FILE_TEMP_EXT, strZ(fileName));
 
+        TEST_ASSIGN(file, storageNewWriteP(storageTest, fileName, .user = STRDEF("root")), "new write file (defaults)");
+        TEST_ERROR(
+            ioWriteOpen(storageWriteIo(file)), FileOwnerError,
+            "unable to set ownership for '" TEST_PATH "/sub1/testfile-abort.pgbackrest.tmp' to root[0]:[none] from " TEST_USER "["
+            TEST_USER_ID_Z "]:" TEST_GROUP "[" TEST_GROUP_ID_Z "]: [1] Operation not permitted");
+
         TEST_ASSIGN(
             file, storageNewWriteP(storageTest, fileName, .user = TEST_USER_STR), "new write file (defaults)");
         TEST_RESULT_VOID(ioWriteOpen(storageWriteIo(file)), "open file");
@@ -949,6 +1010,14 @@ testRun(void)
         // -------------------------------------------------------------------------------------------------------------------------
         TEST_TITLE("write file - set mode");
 
+        TEST_ASSIGN(
+            file, storageNewWriteP(storageTest, fileName, .user = TEST_USER_STR, .group = STRDEF("root")),
+            "new write file (defaults)");
+        TEST_ERROR(
+            ioWriteOpen(storageWriteIo(file)), FileOwnerError,
+            "unable to set ownership for '" TEST_PATH "/sub1/testfile-abort.pgbackrest.tmp' to " TEST_USER "[" TEST_USER_ID_Z
+            "]:root[0] from " TEST_USER "[" TEST_USER_ID_Z "]:" TEST_GROUP "[" TEST_GROUP_ID_Z "]: [1] Operation not permitted");
+
         fileName = STRDEF(TEST_PATH "/sub2/testfile");
 
         TEST_ASSIGN(
@@ -1055,8 +1124,8 @@ testRun(void)
         TEST_TITLE("read offset/limited bytes");
 
         TEST_ASSIGN(
-            buffer, storageGetP(storageNewReadP(storageTest, STRDEF(TEST_PATH "/test.txt"), .offset = 4,
-            .limit = VARUINT64(4))), "get");
+            buffer,
+            storageGetP(storageNewReadP(storageTest, STRDEF(TEST_PATH "/test.txt"), .offset = 4, .limit = VARUINT64(4))), "get");
         TEST_RESULT_UINT(bufSize(buffer), 4, "check size");
         TEST_RESULT_BOOL(memcmp(bufPtrConst(buffer), "FILE", bufSize(buffer)) == 0, true, "check content");
     }
@@ -1231,6 +1300,7 @@ testRun(void)
         TEST_RESULT_STR(storageWriteName(file), fileNoPerm, "check name");
         TEST_RESULT_BOOL(storageWriteSyncPath(file), false, "check sync path");
         TEST_RESULT_BOOL(storageWriteSyncFile(file), false, "check sync file");
+        TEST_RESULT_BOOL(storageWriteTruncate(file), true, "file will be truncated");
 
         // -------------------------------------------------------------------------------------------------------------------------
         TEST_TITLE("permission denied");
@@ -1345,6 +1415,24 @@ testRun(void)
         TEST_RESULT_INT(storageInfoP(storageTest, fileName).mode, 0600, "check file mode");
 
         storageRemoveP(storageTest, fileName, .errorOnMissing = true);
+
+        // -------------------------------------------------------------------------------------------------------------------------
+        TEST_TITLE("no truncate");
+
+        HRN_STORAGE_PUT_Z(storageTest, "no-truncate", "ABC", .modeFile = 0600);
+
+        TEST_ASSIGN(
+            file,
+            storageNewWriteP(
+                storageTest, STRDEF("no-truncate"), .modeFile = 0660, .timeModified = 77777, .noAtomic = true, .noTruncate = true),
+            "new write file");
+        TEST_RESULT_BOOL(storageWriteTruncate(file), false, "file will not be truncated");
+        TEST_RESULT_VOID(ioWriteOpen(storageWriteIo(file)), "open file");
+        TEST_RESULT_VOID(ioWriteClose(storageWriteIo(file)), "close file");
+
+        TEST_STORAGE_GET(storageTest, "no-truncate", "ABC");
+        TEST_RESULT_UINT(storageInfoP(storageTest, STRDEF("no-truncate")).mode, 0600, "check mode");
+        TEST_RESULT_INT(storageInfoP(storageTest, STRDEF("no-truncate")).timeModified, 77777, "check time");
     }
 
     // *****************************************************************************************************************************
